import csv
import argparse
import json
import datetime

PREROLL_TIME = 5 

def duplicate_and_insert_first_row(file_path, output_path, rate, add_steps):
    time_step_ns = int((1.0 / rate) * 1e9)  # Convert rate to nanoseconds
    total_time_ns = PREROLL_TIME * 1e9  # e.g. 10 seconds in nanoseconds
    num_duplicates = int(total_time_ns / time_step_ns)
    
    with open(file_path, 'r', newline='') as f:
        reader = csv.reader(f)
        rows = list(reader)
    
    if not rows:
        print("Error: CSV file is empty.")
        return
    
    header = rows[0]
    if len(rows) < 2:
        print("Error: CSV file does not contain enough data rows.")
        return
    
    first_row = rows[1]
    timestamp_index = header.index("timestamp(ns)")
    first_timestamp = int(first_row[timestamp_index])
    
    new_rows = []
    batch = 1
    idx = 1
    for i in range(num_duplicates, 0, -1):
        new_row = first_row[:]
        new_row[timestamp_index] = str(first_timestamp - (i * time_step_ns))
        if add_steps:
            new_row[0] = idx 
            new_row[2] = batch
            new_row[3] = 0.0
            new_row[4] = 0.0
            new_row[5] = -9.8
            new_row[6] = 0.0
            new_row[7] = 0.0
            new_row[8] = 0.0
            idx = idx + 1
            if idx % 32 == 0:
                batch = batch + 1

        new_rows.append(new_row)


    
    batch = batch + 1
    for row in rows[1:]:
        row[0] = idx
        row[2] = batch
        idx = idx + 1
        if idx % 32 == 0:
            batch = batch + 1

    
    with open(output_path, 'w', newline='') as f:
        writer = csv.writer(f)
        writer.writerow(header)
        writer.writerows(new_rows + rows[1:])
    
    print(f"Modified file saved as: {output_path}")

def modify_json_data(filepath):
    with open(filepath, 'r') as file:
        data = json.load(file)

    if "start_time_monotonic_ns" in data:
        data["start_time_monotonic_ns"] -= int(PREROLL_TIME * 1e9)  # Subtract 5e9 (5 billion)

    if "start_time_date" in data:
        date_str = data["start_time_date"]
        dt = datetime.datetime.strptime(date_str, "%Y-%m-%d %H:%M:%S") #parse with specific format
        modified_dt = dt + datetime.timedelta(seconds=PREROLL_TIME)
        data["start_time_date"] = modified_dt.strftime("%Y-%m-%d %H:%M:%S") #format back to string

    if "duration_s" in data:
        data["duration_s"] += PREROLL_TIME 

    if "n_channels" in data:
        data["n_channels"] = 3

    data = filter_channels(data)

    with open(filepath, 'w') as file:  # Open the same file for writing
        json.dump(data, file, indent=4)  # Save the modified data

# front/down/imu only
def filter_channels(data):
    filtered_channels = []
    for channel in data["channels"]:
        if channel.get("channel") in [0, 1, 5]:
            channel["n_samples"] += 500
            filtered_channels.append(channel) 
      
    data["channels"] = filtered_channels
     
    return data


def main():
    parser = argparse.ArgumentParser(description="Duplicate and insert first row with adjusted timestamps.")
    parser.add_argument("input_file", type=str, help="Path to the input CSV file")
    parser.add_argument("output_file", type=str, help="Path to save the modified CSV file")
    parser.add_argument("rate", type=float, help="Rate in Hz to determine time step in nanoseconds")
    parser.add_argument("add", type=int, help="add time steps")
    parser.add_argument("json", type=str, help="json file to modify")
    args = parser.parse_args()
    
    duplicate_and_insert_first_row(args.input_file, args.output_file, args.rate, args.add)
    modify_json_data(args.json)


if __name__ == "__main__":
    main()
